SystemOrganization addCategory: #'MethodWrappers-Tests'!
SystemOrganization addCategory: #'MethodWrappers-Base'!
SystemOrganization addCategory: #'MethodWrappers-Examples'!

!CompiledMethod methodsFor: '*MethodWrappers' stamp: 'bs 10/27/2009 21:55'!
clientMethod

	^ self! !

!CompiledMethod methodsFor: '*MethodWrappers' stamp: 'rhi 4/29/2001 09:33'!
unwrappedMethod

	^ self! !

CompiledMethod variableByteSubclass: #MwMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Base'!

MwMethodWrapper class
	instanceVariableNames: 'protoMethods'!

!MwMethodWrapper commentStamp: '<historical>' prior: 0!
Method wrappers can add "hidden" behavior to the ordinary method. They can change what is executed before or after the method. For example, a count method wrapper would increment a counter when the method is called, and then forward the call onto the original method.

While this class is not abstract (i.e., there are no subclassResponsibilities), these method wrappers don't do anything if you install them. Look at the subclasses for example uses.

To create a new type of method wrapper, you must create a subclass of MwMethodWrapper and redefine the #beforeMethod and/or #afterMethod as necessary. If you only need to redefine the #beforeMethod, you can redefine the #valueFrom:arguments: method instead (see the count method wrapper). This will give you some added performance since it doesn't need to create two blocks and send the #valueNowOrOnUnwindDo: message.

To use a method wrapper, send the class a #on:inClass: message. This will return a new method wrapper (or nil if this method shouldn't be wrapped). You can then install the method wrapper by sending the #install message and uninstall by sending the #uninstall message.

WARNING: Incorrectly using the wrappers can crash your image, so you should save your image before using them. Most problems occur as a result of installing them on methods that are called either in the #beforeMethod or #afterMethod. (Note: The #value method in Association (VariableBinding in VW2.5) is automatically called by the VM when you refer to a global, class, or pool variable so you should probably never wrap it. I advise against wrapping primitives also.)

Class Instance Variables:

	protoMethods		<Dictionary
								key: SmallInteger
								value: MwMethodWrapper>
							Prototype wrappers that are copied when installing new wrappers.
	wrapperInstVars	<WeakIdentityKeyDictionary
								key: MwMethodWrapper
								value: MwMethodWrapperInstVars>
							A kludge to add instance varibles to non-pointer classes.!

MwMethodWrapper class
	instanceVariableNames: 'protoMethods'!

MwMethodWrapper variableByteSubclass: #MwBlockHandlerMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Base'!

!MwBlockHandlerMethodWrapper commentStamp: '<historical>' prior: 0!
Inherited Class Instance Variables:

	wrapperInstVars	<WeakIdentityKeyDictionary
								key: MwBlockHandlerMethodWrapper
								value: MwBlockHandlerMethodWrapperInstVars>
							A kludge to add instance varibles to non-pointer classes.!

!MwBlockHandlerMethodWrapper class methodsFor: 'class initialization' stamp: 'rhi 4/7/2001 14:55'!
initialize
	"doIt: [self initialize]"

	super initialize.! !

!MwBlockHandlerMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 4/7/2001 20:20'!
afterMethod

	self shouldNotImplement.! !

!MwBlockHandlerMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 4/7/2001 20:20'!
beforeMethod

	self shouldNotImplement.! !

!MwBlockHandlerMethodWrapper methodsFor: 'initialization' stamp: 'rhi 6/17/2001 21:48'!
class: aClass selector: aSymbol

	super class: aClass selector: aSymbol.
	self
		exception: Error;
		handlerBlock: [:receiver :arguments :ex | ].! !

!MwBlockHandlerMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:50'!
exception
	^ self properties at: #mwException ifAbsent: [].! !

!MwBlockHandlerMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:51'!
exception: anExceptionClass
	self properties at: #mwException put: anExceptionClass! !

!MwBlockHandlerMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/27/2009 21:50'!
handlerBlock
	^ self properties at: #mwHandlerBlock ifAbsent: [nil].! !

!MwBlockHandlerMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:52'!
handlerBlock: aBlockContext
	self properties at: #mwHandlerBlock put: aBlockContext fixTemps.! !

!MwBlockHandlerMethodWrapper methodsFor: 'private' stamp: 'rhi 4/7/2001 20:17'!
instVarsClass

	^ MwBlockHandlerMethodWrapperInstVars! !

!MwBlockHandlerMethodWrapper methodsFor: 'installation' stamp: 'bs 12/18/2009 14:22'!
uninstall

	self handlerBlock ifNotNilDo: [:block | block instVarNamed: 'outerContext' put: nil].
	self flag: #rhi. "self handlerBlock: nil."
	super uninstall.! !

!MwBlockHandlerMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 6/17/2001 21:47'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	^ [
		self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
	] on: self exception do: [:ex |
		self handlerBlock copy fixTemps value: anObject value: anArrayOfObjects value: ex]! !

MwMethodWrapper variableByteSubclass: #MwBlockMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Base'!

!MwBlockMethodWrapper commentStamp: '<historical>' prior: 0!
Inherited Class Instance Variables:

	wrapperInstVars	<WeakIdentityKeyDictionary
								key: MwBlockMethodWrapper
								value: MwBlockMethodWrapperInstVars>
							A kludge to add instance varibles to non-pointer classes.!

!MwBlockMethodWrapper class methodsFor: 'class initialization' stamp: 'rhi 3/30/2001 10:03'!
initialize
	"doIt: [self initialize]"

	super initialize.! !

!MwBlockMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:52'!
afterBlock
	^ self properties at: #mwAfterBlock ifAbsent: []! !

!MwBlockMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:58'!
afterBlock: aBlockContext

	self properties at: #mwAfterBlock put: aBlockContext fixTemps.! !

!MwBlockMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 10:37'!
afterMethod

	self shouldNotImplement.! !

!MwBlockMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:53'!
beforeBlock

	^ self properties at: #mwBeforeBlock ifAbsent: []! !

!MwBlockMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:53'!
beforeBlock: aBlockContext
	self properties at: #mwBeforeBlock put: aBlockContext fixTemps! !

!MwBlockMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 10:37'!
beforeMethod

	self shouldNotImplement.! !

!MwBlockMethodWrapper methodsFor: 'initialization' stamp: 'rhi 4/19/2001 09:51'!
class: aClass selector: aSymbol

	super class: aClass selector: aSymbol.
	self
		beforeBlock: [:receiver :arguments | ];
		afterBlock: [:receiver :arguments | ].! !

!MwBlockMethodWrapper methodsFor: 'private' stamp: 'rhi 3/30/2001 09:52'!
instVarsClass

	^ MwBlockMethodWrapperInstVars! !

!MwBlockMethodWrapper methodsFor: 'installation' stamp: 'bs 12/18/2009 14:21'!
uninstall

	self flag: #rhi. "self clientMethod ifNil: [^ self]."
	self beforeBlock ifNotNilDo: [:block | block instVarNamed: 'outerContext' put: nil].
	self flag: #rhi. "self beforeBlock: nil."
	self afterBlock ifNotNilDo: [:block | block instVarNamed: 'outerContext' put: nil].
	self flag: #rhi. "self afterBlock: nil."
	super uninstall.! !

!MwBlockMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 6/17/2001 21:53'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	"This is the general case where you want both a before and after block, but if you
	want just a before block, you might want to override this method for optimization."

	self beforeBlock copy fixTemps value: anObject value: anArrayOfObjects.
	^ [
		self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
	] ensure: [self afterBlock copy fixTemps value: anObject value: anArrayOfObjects]! !

MwMethodWrapper variableByteSubclass: #MwCalledMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

MwCalledMethodWrapper class
	instanceVariableNames: 'installing'!

!MwCalledMethodWrapper commentStamp: '<historical>' prior: 0!
A called method wrapper is a special method wrapper that uninstalls itself once it has been called. Although this is initially slower than the count method wrapper, it will speed up to be the same speed as the unwrapped code.

Class Instance Variables:

	installing	<Boolean>
					Are we installing the wrappers? During installation, some of the
					methods might be called, but we shouldn't tag them as being called,
					since it results from the installation process, not normal code execution.

Inherited Class Instance Variables:

	wrapperInstVars	<WeakIdentityKeyDictionary
								key: MwCalledMethodWrapper
								value: MwCalledMethodWrapperInstVars>
							A kludge to add instance varibles to non-pointer classes.!

MwCalledMethodWrapper class
	instanceVariableNames: 'installing'!

!MwCalledMethodWrapper class methodsFor: 'class initialization' stamp: 'rhi 3/30/2001 14:09'!
initialize
	"doIt: [self initialize]"

	super initialize.
	self installing: false.! !

!MwCalledMethodWrapper class methodsFor: 'accessing' stamp: 'rhi 3/30/2001 14:10'!
installing
	"^ <Boolean>"

	^ installing! !

!MwCalledMethodWrapper class methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
installing: aBoolean

	installing := aBoolean.! !

!MwCalledMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 18:49'!
afterMethod

	self called ifTrue: [self uninstall].! !

!MwCalledMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 18:48'!
beforeMethod

	(self called or: [self installing])
		ifFalse: [self called: true].! !

!MwCalledMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:54'!
called

	^ self properties at: #mwCalled ifAbsent: []! !

!MwCalledMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:54'!
called: aBoolean

	self properties at: #mwCalled put: aBoolean.! !

!MwCalledMethodWrapper methodsFor: 'initialization' stamp: 'rhi 3/30/2001 14:06'!
class: aClass selector: aSymbol

	self called: false.
	^ super class: aClass selector: aSymbol! !

!MwCalledMethodWrapper methodsFor: 'private' stamp: 'rhi 3/30/2001 14:48'!
instVarsClass

	^ MwCalledMethodWrapperInstVars! !

!MwCalledMethodWrapper methodsFor: 'derived accessing' stamp: 'rhi 3/21/2001 12:56'!
installing

	^ self class installing! !

MwMethodWrapper variableByteSubclass: #MwCountMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

!MwCountMethodWrapper commentStamp: '<historical>' prior: 0!
A count method wrapper counts the number of times the method is called.

Inherited Class Instance Variables:

	wrapperInstVars	<WeakIdentityKeyDictionary
								key: MwCountMethodWrapper
								value: MwCountMethodWrapperInstVars>
							A kludge to add instance varibles to non-pointer classes.!

!MwCountMethodWrapper class methodsFor: 'class initialization' stamp: 'rhi 3/30/2001 13:51'!
initialize
	"doIt: [self initialize]"

	super initialize.! !

!MwCountMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 6/18/2001 12:06'!
beforeMethod

	self count: self count + 1.! !

!MwCountMethodWrapper methodsFor: 'initialization' stamp: 'rhi 3/21/2001 15:07'!
class: aClass selector: aSymbol

	self count: 0.
	^ super class: aClass selector: aSymbol! !

!MwCountMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:55'!
count

	^ self properties at: #mwCount ifAbsent: []! !

!MwCountMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:56'!
count: anInteger

	self properties at: #mwCount put: anInteger.! !

!MwCountMethodWrapper methodsFor: 'private' stamp: 'rhi 3/30/2001 13:42'!
instVarsClass

	^ MwCountMethodWrapperInstVars! !

MwCountMethodWrapper variableByteSubclass: #MwTimeMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

!MwTimeMethodWrapper commentStamp: '<historical>' prior: 0!
A time method wrapper counts the number of times the method has been called, along with the total execution time this method has taken.

Inherited Class Instance Variables:

	wrapperInstVars	<WeakIdentityKeyDictionary
								key: MwTimeMethodWrapper
								value: MwTimeMethodWrapperInstVars>
							A kludge to add instance varibles to non-pointer classes.!

!MwTimeMethodWrapper class methodsFor: 'class initialization' stamp: 'rhi 3/30/2001 13:55'!
initialize
	"doIt: [self initialize]"

	super initialize.! !

!MwTimeMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 19:03'!
afterMethod

	self shouldNotImplement.! !

!MwTimeMethodWrapper methodsFor: 'derived accessing' stamp: 'rhi 2/23/2001 14:34'!
averageTime

	^ self count == 0
		ifTrue: [0]
		ifFalse: [self totalTime / self count asFloat]! !

!MwTimeMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 19:03'!
beforeMethod

	self shouldNotImplement.! !

!MwTimeMethodWrapper methodsFor: 'initialization' stamp: 'rhi 3/21/2001 15:22'!
class: aClass selector: aSymbol

	self time: 0.
	^ super class: aClass selector: aSymbol! !

!MwTimeMethodWrapper methodsFor: 'private' stamp: 'rhi 3/30/2001 13:43'!
instVarsClass

	^ MwTimeMethodWrapperInstVars! !

!MwTimeMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:56'!
time

	^ self properties at: #mwTime! !

!MwTimeMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:56'!
time: anInteger

	self properties at: #mwTime put: anInteger.! !

!MwTimeMethodWrapper methodsFor: 'derived accessing' stamp: 'rhi 3/21/2001 15:20'!
totalTime

	^ self time! !

!MwTimeMethodWrapper methodsFor: 'evaluating' stamp: 'DF 4/19/2006 00:00'!
valueWithReceiver: anObject arguments: argumentsArray
	"This method was overriden so that recursive calls will have a different beginTime."

	| beginTime |
	beginTime := Time millisecondClockValue.
	self count: self count + 1.
	^ [
		self clientMethod valueWithReceiver: anObject arguments: argumentsArray
	] ensure: [self time: self time + (Time millisecondClockValue - beginTime max: 0)]! !

!MwMethodWrapper class methodsFor: 'monticello compatibility' stamp: 'DF 4/19/2006 01:18'!
asClassDefinition

	| mcClassDefinition |
	
	mcClassDefinition := Smalltalk at: #MCClassDefinition ifAbsent: [^self error: 'This message is implemented for Monticello compatibility, but MCClassDefinition is not installed in the image. Possible causes: MC is not installed or you are using a different version of MC' ].
	
	^mcClassDefinition
		name: self name
		superclassName: self superclass name
		traitComposition: self traitComposition asString
		classTraitComposition: self class traitComposition asString
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self class instVarNames
		type: #bytes
		comment: self organization classComment	 asString
		commentStamp: self organization commentStamp! !

!MwMethodWrapper class methodsFor: 'testing' stamp: 'DF 4/19/2006 00:00'!
canWrap: aSelector inClass: aClass
	"Test if a method can be wrapped without causing infinite recursion."

	| class method |
	(aClass includesBehavior: MwMethodWrapper) ifTrue: [^ false].
	((aClass includesBehavior: CompiledMethod)
		and: [aSelector == #valueWithReceiver:arguments:])
			ifTrue: [^ false].
	((aClass includesBehavior: BlockContext)
		and: [aSelector == #ensure:])
			ifTrue: [^ false].
	class := aClass whichClassIncludesSelector: aSelector.
	class isNil ifTrue: [^ false].
	method := class compiledMethodAt: aSelector ifAbsent: [nil].
	^ method notNil and: [(self primitives includes: method primitive) not]! !

!MwMethodWrapper class methodsFor: 'CHECKS' stamp: 'DF 4/19/2006 00:00'!
checkClassFormats
	"doIt: [self checkClassFormats]"

	| allFormatsOk |
	allFormatsOk := true.
	Smalltalk garbageCollect.
	Transcript cr.
	self withAllSubclassesDo: [:each |
		| formatOk |
		formatOk := (each format bitAnd: 16r200) = 16r200.
		allFormatsOk := allFormatsOk & formatOk.
		Transcript cr; show: each name, ' -> ', formatOk printString].
	Transcript cr; show: '=== ALL FORMATS OK -> ', allFormatsOk printString.! !

!MwMethodWrapper class methodsFor: 'private' stamp: 'rhi 2/26/2001 08:53'!
codeStringFor: numArgs

	| nameString tempsString |
	numArgs == 0
		ifTrue: [
			nameString _ 'value'.
			tempsString _ 't _ #Array new']
		ifFalse: [
			nameString _ ''.
			tempsString _ '(t _ #Array new: ', numArgs printString, ')'.
			1 to: numArgs do: [:i |
				nameString _ nameString,
					'value: t',
					i printString,
					(i == numArgs ifTrue: [''] ifFalse: [' ']).
				tempsString _ tempsString,
					(i == 1 ifTrue: [''] ifFalse: [';']),
					' at: ' , i printString,
					' put: t',
					i printString]].
	^ nameString, '
	| t |
	', tempsString, '.
	^ ',
	self methodWrapperSymbol printString,
	' valueWithReceiver: self arguments: t'! !

!MwMethodWrapper class methodsFor: 'CHECKS' stamp: 'DF 4/19/2006 00:00'!
countAllInstances
	"doIt: [self countAllInstances]"

	| totalCount |
	totalCount := 0.
	Smalltalk garbageCollect.
	Transcript cr.
	self withAllSubclassesDo: [:each |
		| count |
		count := each allInstances size.
		totalCount := totalCount + count.
		Transcript cr; show: each name, ' -> ', count printString].
	Transcript cr; show: '=== TOTAL COUNT -> ', totalCount printString.! !

!MwMethodWrapper class methodsFor: 'private' stamp: 'DF 4/18/2006 23:58'!
createMethodFor: numArgs

	^ (Compiler new
			compile: (self codeStringFor: numArgs)
			in: self
			notifying: nil
			ifFail: [])
		generateWith: #(0 0 0 0) using: self! !

!MwMethodWrapper class methodsFor: 'class initialization' stamp: 'bs 10/16/2009 10:02'!
initialize
	"doIt: [self initialize]"

	"--------------------------------------------------------------------------------"
	self tweakClassFormat.
	"--------------------------------------------------------------------------------"
	self protoMethods: Dictionary new.! !

!MwMethodWrapper class methodsFor: 'class initialization' stamp: 'rhi 3/24/2001 08:07'!
initializeWithAllSubclasses
	"doIt: [self initializeWithAllSubclasses]"

	self withAllSubclassesDo: [:each | each initialize].! !

!MwMethodWrapper class methodsFor: 'private' stamp: 'rhi 3/21/2001 19:09'!
methodWrapperSymbol

	^ #'The method wrapper should be inserted in this position!!'! !

!MwMethodWrapper class methodsFor: 'private' stamp: 'rhi 3/16/2001 23:27'!
nuke
	"doIt: [self nuke]"

	self
		uninstallAllWrappers;
		initialize.! !

!MwMethodWrapper class methodsFor: 'instance creation' stamp: 'bs 10/16/2009 08:58'!
on: aSymbol inClass: aClass

	| protoMethod wrapper |
	self flag: #rhi. "How about signaling an exception instead of returning nil?"
	(self canWrap: aSymbol inClass: aClass) ifFalse: [^ nil].
	protoMethod := (self protoMethods
			at: aSymbol numArgs
			ifAbsentPut: [self createMethodFor: aSymbol numArgs]).
	wrapper := protoMethod copyWithTrailerBytes: #(0 0 0 0).
	wrapper 
		properties: protoMethod properties copy;
		class: aClass selector: aSymbol.
	^ wrapper! !

!MwMethodWrapper class methodsFor: 'private' stamp: 'rhi 2/25/2001 21:31'!
primitives

	^ #(
		61		"at:put:"
		71		"new:"
		81		"value"
	)! !

!MwMethodWrapper class methodsFor: 'accessing' stamp: 'rhi 3/21/2001 18:20'!
protoMethods
	"^ <Dictionary key: SmallInteger value: MwMethodWrapper>"

	^ protoMethods! !

!MwMethodWrapper class methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
protoMethods: aDictionary

	protoMethods := aDictionary.! !

!MwMethodWrapper class methodsFor: 'class initialization' stamp: 'DF 4/19/2006 00:34'!
tweakClassFormat

	self setFormat: (self format bitOr: 16r200).! !

!MwMethodWrapper class methodsFor: 'private' stamp: 'rhi 6/18/2001 14:41'!
uninstallAllWrappers
	"doIt: [self uninstallAllWrappers]"

	self allSubInstancesDo: [:inst | inst uninstall].! !

!MwMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/19/2001 21:17'!
afterMethod! !

!MwMethodWrapper methodsFor: 'private' stamp: 'DF 4/19/2006 00:00'!
arrayPosition

	^ (2 to: self numLiterals + 1) detect: [:i |
			| each |
			each := self realObjectAt: i.
			each == #Array or: [each == Array]]
		ifNone: [self error: 'Cannot find position for Array']! !

!MwMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 3/19/2001 21:17'!
beforeMethod! !

!MwMethodWrapper methodsFor: 'initialization' stamp: 'rhi 3/16/2001 23:26'!
class: aClass selector: aSymbol

	self
		objectAt: self methodPosition put: self;
		objectAt: self arrayPosition put: Array;
		wrappedClass: aClass;
		clientSelector: aSymbol.! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:16'!
clientMethod

	^ self properties at: #mwClientMethod ifAbsent: []! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:17'!
clientMethod: aCompiledMethod

	self properties at: #mwClientMethod put: aCompiledMethod.! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:17'!
clientSelector

	^ self properties at: #mwClientSelector ifAbsent: []! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:18'!
clientSelector: aSymbol

	self properties at: #mwClientSelector put: aSymbol.! !

!MwMethodWrapper methodsFor: 'initialization' stamp: 'DF 4/19/2006 00:00'!
copyWithTrailerBytes: bytes

	| start end copy |
	start := self initialPC.
	end := self endPC.
	copy := self class newMethod: end - start + 1 + bytes size header: self header.
	1 to: self numLiterals do: [:i | copy literalAt: i put: (self literalAt: i)].
	start to: end do: [:i | copy at: i put: (self at: i)].
	1 to: bytes size do: [:i | copy at: end + i put: (bytes at: i)].
	^ copy! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:18'!
definingClass

	^ self properties at: #mwDefiningClass ifAbsent: []! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:19'!
definingClass: aClass

	self properties at: #mwDefiningClass put: aClass.! !

!MwMethodWrapper methodsFor: 'private' stamp: 'rhi 3/30/2001 10:08'!
instVarsClass

	^ MwMethodWrapperInstVars! !

!MwMethodWrapper methodsFor: 'installation' stamp: 'bs 10/28/2009 09:19'!
install

	| holdingClass method |
	self flag: #rhi. "KLUDGE: Einwegwrapper."
	self wasActive ifTrue: [self error: 'Einwegwrapper!!'].
	holdingClass := self wrappedClass whichClassIncludesSelector: self clientSelector.
	holdingClass isNil ifTrue: [^ self].
	method := holdingClass compiledMethodAt: self clientSelector.
	method == self ifTrue: [^ self].
	self
		clientMethod: method;
		setSourcePointer: method sourcePointer.
	method isMwMethodWrapper
		ifTrue: [
			method wrappingWrapper: self.
			self definingClass: method definingClass]
		ifFalse: [self definingClass: holdingClass].
	self wrappedClass addSelectorSilently: self clientSelector withMethod: self.! !

!MwMethodWrapper methodsFor: 'testing' stamp: 'rhi 4/9/2001 21:21'!
isMwMethodWrapper

	^ true! !

!MwMethodWrapper methodsFor: 'private' stamp: 'DF 4/19/2006 00:00'!
methodPosition

	^ (2 to: self numLiterals + 1) detect: [:i |
			| each |
			each := self realObjectAt: i.
			each == self class methodWrapperSymbol or: [each == self]]
		ifNone: [self error: 'Cannot find position to insert wrapper']! !

!MwMethodWrapper methodsFor: 'private' stamp: 'DF 4/19/2006 00:00'!
objectAt: anIndex 
	"Lie about what's in our literal frame. This has the bad side-effect that we can't
	view the correct information in inspectors, but the good news is that we stop
	infinite recursion. Good thing the VM doesn't use the basicAt: method :)."

	| object |
	object := self realObjectAt: anIndex.
	^ object == self ifTrue: [self clientMethod] ifFalse: [object]! !

!MwMethodWrapper methodsFor: 'private' stamp: 'rhi 2/26/2001 09:03'!
realObjectAt: anIndex

	^ super objectAt: anIndex! !

!MwMethodWrapper methodsFor: 'installation' stamp: 'bs 10/28/2009 09:19'!
uninstall

	self flag: #rhi. "The #uninstall should actually become #destroy!!"
	[	self clientMethod ifNil: [^ self].
		(self wrappedClass compiledMethodAt: self clientSelector ifAbsent: [nil]) == self
			ifTrue: [self wrappedClass == self definingClass
				ifTrue: [self wrappedClass
					addSelectorSilently: self clientSelector
					withMethod: self clientMethod]
				ifFalse: [self wrappedClass removeSelectorSilently: self clientSelector]].
		self clientMethod isMwMethodWrapper
			ifTrue: [self clientMethod wrappingWrapper: self wrappingWrapper].
		self wrappingWrapper
			ifNotNil: [
				self wrappingWrapper
					clientMethod: self clientMethod;
					definingClass: self definingClass.
				self wrappingWrapper: nil].
		self
			definingClass: nil;
			clientMethod: nil.
	] ensure: [
		self flag: #rhi. "KLUDGE: Einwegwrapper."
		self wasActive: true].! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'rhi 2/26/2001 21:28'!
unwrappedMethod

	^ self clientMethod ifNotNil: [self clientMethod unwrappedMethod]! !

!MwMethodWrapper methodsFor: 'evaluating' stamp: 'rhi 4/8/2001 18:26'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	"This is the general case where you want both a before and after method, but if you
	want just a before method, you might want to override this method for optimization."

	self beforeMethod.
	^ [
		self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
	] ensure: [self afterMethod]! !

!MwMethodWrapper methodsFor: 'testing' stamp: 'bs 10/15/2009 16:25'!
wasActive

	self flag: #rhi. "KLUDGE: Einwegwrapper."
	^ self properties at: #mwWasActive ifAbsentPut: [false]! !

!MwMethodWrapper methodsFor: 'testing' stamp: 'bs 10/15/2009 16:19'!
wasActive: aBoolean
	self properties at: #mwWasActive put: aBoolean! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:25'!
wrappedClass

	^ self properties at: #mwWrappedClass ifAbsent: [].! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:25'!
wrappedClass: aClass

	self properties at: #mwWrappedClass put: aClass.! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'rhi 3/30/2001 09:36'!
wrapperInstVars

	^ self class wrapperInstVars! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:25'!
wrappingWrapper

	^ self properties at: #mwWrappingWrapper ifAbsent: []. ! !

!MwMethodWrapper methodsFor: 'derived accessing' stamp: 'bs 10/15/2009 11:26'!
wrappingWrapper: aMwMethodWrapper

	self properties at: #mwWrappingWrapper put: aMwMethodWrapper.! !

MwMethodWrapper variableByteSubclass: #MwSynchronizationWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

!MwSynchronizationWrapper commentStamp: '<historical>' prior: 0!
A synchronization wrapper synchronizes method calls by using a recursion lock. The call to the original method is protected by the RecursionWrapper>>critical: method.

Inherited Class Instance Variables:

	wrapperInstVars	<WeakIdentityKeyDictionary
								key: MwSynchronizationWrapper
								value: MwSynchronizationWrapperInstVars>
							A kludge to add instance varibles to non-pointer classes.!

!MwSynchronizationWrapper class methodsFor: 'testing' stamp: 'rhi 3/27/2001 09:58'!
canWrap: aSelector inClass: aClass

	self flag: #rhi. "Verify that Object>>primitiveChangeClassTo: will not fail!!"
	"^ super canWrap: aSelector inClass: aClass"

	"This could cause infinite recursion if wrapping classes."
	^ true! !

!MwSynchronizationWrapper class methodsFor: 'class initialization' stamp: 'rhi 3/30/2001 14:55'!
initialize
	"doIt: [self initialize]"

	super initialize.! !

!MwSynchronizationWrapper class methodsFor: 'installation' stamp: 'DF 4/19/2006 00:00'!
installOn: aClass

	| lock wrappers |
	lock := Semaphore new.
	self flag: #rhi. "Shouldn't that be #allSelectors?!!"
	wrappers := aClass selectors asOrderedCollection collect: [:each |
		(self on: each inClass: aClass)
			recursionLock: lock;
			install;
			yourself].
	^ wrappers! !

!MwSynchronizationWrapper class methodsFor: 'installation' stamp: 'DF 4/19/2006 00:00'!
installOnObject: anObject

	| oldClass newBehavior lock |
	oldClass := anObject class.
	newBehavior := Behavior new.
	newBehavior instVarNamed: 'format' put: oldClass format.
	newBehavior superclass: oldClass.
	newBehavior methodDictionary: MethodDictionary new.
	self flag: #rhi. "Call self>>#installOn: instead!! (#allSelectors vs. #selectors?!!)"
	lock := Semaphore new.
	newBehavior allSelectors asOrderedCollection collect: [:each |
		(self on: each inClass: newBehavior)
			recursionLock: lock;
			install;
			yourself].
	newBehavior
		addSelector: #synchronize
		withMethod: (self class compiledMethodAt: #synchronize).
	newBehavior
		addSelector: #unsynchronize
		withMethod: (self class compiledMethodAt: #unsynchronize).
	anObject primitiveChangeClassTo: newBehavior basicNew.! !

!MwSynchronizationWrapper class methodsFor: 'private' stamp: 'rhi 3/27/2001 09:49'!
synchronize! !

!MwSynchronizationWrapper class methodsFor: 'installation' stamp: 'DF 4/19/2006 00:00'!
uninstallOn: aClass

	aClass selectors do: [:each |
			| method |
			method := aClass compiledMethodAt: each.
			method isMwMethodWrapper ifTrue: [method uninstall]].! !

!MwSynchronizationWrapper class methodsFor: 'installation' stamp: 'rhi 3/27/2001 09:47'!
uninstallOnObject: anObject

	anObject primitiveChangeClassTo: anObject class superclass basicNew.! !

!MwSynchronizationWrapper class methodsFor: 'private' stamp: 'rhi 3/30/2001 15:36'!
unsynchronize

	self uninstallOnObject: self.! !

!MwSynchronizationWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 19:09'!
afterMethod

	self shouldNotImplement.! !

!MwSynchronizationWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 19:09'!
beforeMethod

	self shouldNotImplement.! !

!MwSynchronizationWrapper methodsFor: 'private' stamp: 'rhi 3/30/2001 14:50'!
instVarsClass

	^ MwSynchronizationWrapperInstVars! !

!MwSynchronizationWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:57'!
recursionLock

	^ self properties at: #mwRecursionLock! !

!MwSynchronizationWrapper methodsFor: 'derived accessing' stamp: 'bs 10/16/2009 09:57'!
recursionLock: aSemaphore

	self properties at: #mwRecursionLock put: aSemaphore.! !

!MwSynchronizationWrapper methodsFor: 'evaluating' stamp: 'rhi 3/30/2001 17:36'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	self flag: #rhi. "Need to ensure RecursionLock semantics!!"
	^ self recursionLock critical: [
		self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects]! !

!MethodNode methodsFor: '*MethodWrappers' stamp: 'bs 12/17/2009 17:44'!
generateWith: trailer using: aCompiledMethodClass
	"The receiver is the root of a parse tree. Answer a CompiledMethod. The
	argument, trailer, is the references to the source code that is stored with 
	every CompiledMethod."

	| literals blkSize method nArgs nLits primErrNode stack strm |
	self generate: trailer ifQuick: 
		[:m |
		literals := encoder allLiterals.
		(nLits := literals size) > 255 ifTrue:
			[^self error: 'Too many literals referenced'].
		1 to: nLits do: [:lit | m literalAt: lit put: (literals at: lit)].
		m properties: properties.
		^m].
	primErrNode := self primitiveErrorVariableName ifNotNil:
						[encoder fixTemp: self primitiveErrorVariableName].
	nArgs := arguments size.
	blkSize := (block sizeForEvaluatedValue: encoder)
				+ (primErrNode ifNil: [0] ifNotNil: [2 "We force store-long (129)"]).
	(nLits := (literals := encoder allLiterals) size) > 255 ifTrue:
		[^self error: 'Too many literals referenced'].
	method := aCompiledMethodClass	"Dummy to allocate right size"
				newBytes: blkSize
				trailerBytes: trailer 
				nArgs: nArgs
				nTemps: encoder maxTemp
				nStack: 0
				nLits: nLits
				primitive: primitive.
	strm := ReadWriteStream with: method.
	strm position: method initialPC - 1.
	stack := ParseStack new init.
	primErrNode ifNotNil: [primErrNode emitStore: stack on: strm].
	block emitForEvaluatedValue: stack on: strm.
	stack position ~= 1 ifTrue:
		[^self error: 'Compiler stack discrepancy'].
	strm position ~= (method size - trailer size) ifTrue:
		[^self error: 'Compiler code size discrepancy'].
	method needsFrameSize: stack size.
	1 to: nLits do: [:lit | method literalAt: lit put: (literals at: lit)].
	method properties: properties.
	^method! !

Object subclass: #MwClassA
	instanceVariableNames: 'x'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwClassA commentStamp: '<historical>' prior: 0!
Instance Variables:

	x	<Integer>!

!MwClassA methodsFor: 'debugging' stamp: 'rhi 6/18/2001 14:17'!
methodDelay

	(Delay forMilliseconds: 1) wait.! !

!MwClassA methodsFor: 'debugging' stamp: 'rhi 6/17/2001 22:48'!
methodOne

	"Transcript cr; show: 'MwClassA>>methodOne'."! !

!MwClassA methodsFor: 'debugging' stamp: 'rhi 4/19/2001 23:33'!
methodTwo

	"Transcript cr; show: 'MwClassA>>methodTwo'."! !

!MwClassA methodsFor: 'debugging' stamp: 'rhi 4/19/2001 13:43'!
methodWithArgument: anInteger

	self x: self x + anInteger.
	^ self x! !

!MwClassA methodsFor: 'debugging' stamp: 'md 10/14/2004 18:55'!
methodWithException

	Warning new  signal: 'Ouch'.! !

!MwClassA methodsFor: 'debugging' stamp: 'rhi 4/19/2001 23:29'!
methodWithoutException

	^ 69! !

!MwClassA methodsFor: 'accessing' stamp: 'rhi 3/30/2001 10:43'!
x
	"^ <Integer>"

	^ x! !

!MwClassA methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
x: anInteger

	x := anInteger.! !

MwClassA subclass: #MwClassB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwClassB methodsFor: 'debugging' stamp: 'rhi 4/19/2001 23:33'!
methodThree

	"Transcript cr; show: 'MwClassB>>methodThree'."! !

!MwClassB methodsFor: 'debugging' stamp: 'rhi 4/19/2001 23:33'!
methodTwo

	"Transcript cr; show: 'MwClassB>>methodTwo'."! !

Object subclass: #MwMethodWrapperInstVars
	instanceVariableNames: 'clientSelector clientMethod definingClass wrappedClass wrappingWrapper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Base'!

!MwMethodWrapperInstVars commentStamp: '<historical>' prior: 0!
Instance Variables:

	clientSelector		<Symbol>
	clientMethod		<CompiledMethod>
	definingClass		<Class>
	wrappedClass		<Class>
	wrappingWrapper	<MwMethodWrapper>!

MwMethodWrapperInstVars subclass: #MwBlockHandlerMethodWrapperInstVars
	instanceVariableNames: 'exception handlerBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Base'!

!MwBlockHandlerMethodWrapperInstVars commentStamp: '<historical>' prior: 0!
Instance Variables:

	exception		<Exception class>
	handlerBlock	<BlockContext>!

!MwBlockHandlerMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 4/7/2001 20:28'!
exception
	"^ <Exception class>"

	^ exception! !

!MwBlockHandlerMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
exception: anExceptionClass

	exception := anExceptionClass.! !

!MwBlockHandlerMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 4/7/2001 15:05'!
handlerBlock
	"^ <BlockContext>"

	^ handlerBlock! !

!MwBlockHandlerMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
handlerBlock: aBlockContext

	handlerBlock := aBlockContext.! !

MwMethodWrapperInstVars subclass: #MwBlockMethodWrapperInstVars
	instanceVariableNames: 'beforeBlock afterBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Base'!

!MwBlockMethodWrapperInstVars commentStamp: '<historical>' prior: 0!
Instance Variables:

	beforeBlock		<BlockContext>
	afterBlock		<BlockContext>!

!MwBlockMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/30/2001 22:38'!
afterBlock
	"^ <BlockContext>"

	^ afterBlock! !

!MwBlockMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
afterBlock: aBlockContext

	afterBlock := aBlockContext.! !

!MwBlockMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/30/2001 22:38'!
beforeBlock
	"^ <BlockContext>"

	^ beforeBlock! !

!MwBlockMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
beforeBlock: aBlockContext

	beforeBlock := aBlockContext.! !

MwMethodWrapperInstVars subclass: #MwCalledMethodWrapperInstVars
	instanceVariableNames: 'called'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

!MwCalledMethodWrapperInstVars commentStamp: '<historical>' prior: 0!
Instance Variables:

	called	<Boolean>!

!MwCalledMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/30/2001 14:02'!
called
	"^ <Boolean>"

	^ called! !

!MwCalledMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
called: aBoolean

	called := aBoolean.! !

MwMethodWrapperInstVars subclass: #MwCountMethodWrapperInstVars
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

!MwCountMethodWrapperInstVars commentStamp: '<historical>' prior: 0!
Instance Variables:

	count	<Integer>!

!MwCountMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/30/2001 13:38'!
count
	"^ <Integer>"

	^ count! !

!MwCountMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
count: anInteger

	count := anInteger.! !

MwCountMethodWrapperInstVars subclass: #MwTimeMethodWrapperInstVars
	instanceVariableNames: 'time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

!MwTimeMethodWrapperInstVars commentStamp: '<historical>' prior: 0!
Instance Variables:

	time	<Integer>!

!MwTimeMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/30/2001 13:39'!
time
	"^ <Integer>"

	^ time! !

!MwTimeMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
time: anInteger

	time := anInteger.! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/17/2001 23:45'!
clientMethod
	"^ <CompiledMethod>"

	^ clientMethod! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
clientMethod: aCompiledMethod

	clientMethod := aCompiledMethod.! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/17/2001 23:45'!
clientSelector
	"^ <Symbol>"

	^ clientSelector! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
clientSelector: aSymbol

	clientSelector := aSymbol.! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/17/2001 23:46'!
definingClass
	"^ <Class>"

	^ definingClass! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
definingClass: aClass

	definingClass := aClass.! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/17/2001 23:46'!
wrappedClass
	"^ <Class>"

	^ wrappedClass! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
wrappedClass: aClass

	wrappedClass := aClass.! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 4/9/2001 15:36'!
wrappingWrapper
	"^ <MwMethodWrapper>"

	^ wrappingWrapper! !

!MwMethodWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
wrappingWrapper: aMwMethodWrapper

	wrappingWrapper := aMwMethodWrapper.! !

MwMethodWrapperInstVars subclass: #MwSynchronizationWrapperInstVars
	instanceVariableNames: 'recursionLock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Examples'!

!MwSynchronizationWrapperInstVars commentStamp: '<historical>' prior: 0!
Instance Variables:

	recursionLock	<Semaphore>!

!MwSynchronizationWrapperInstVars methodsFor: 'accessing' stamp: 'rhi 3/30/2001 14:22'!
recursionLock
	"^ <Semaphore>"

	^ recursionLock! !

!MwSynchronizationWrapperInstVars methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
recursionLock: aSemaphore

	recursionLock := aSemaphore.! !

!Object methodsFor: '*MethodWrappers' stamp: 'rhi 4/9/2001 21:22'!
isMwMethodWrapper

	^ false! !

!Object methodsFor: '*MethodWrappers' stamp: 'rhi 3/27/2001 09:55'!
synchronize

	MwSynchronizationWrapper installOnObject: self.! !

!Object methodsFor: '*MethodWrappers' stamp: 'rhi 3/27/2001 09:54'!
unsynchronize

	^ self! !

TestCase subclass: #MwTestCase
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

MwTestCase subclass: #MwBlockHandlerMethodWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'rhi 6/18/2001 14:32'!
setUp

	self result: 0.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 10:00'!
testClassInstVars

	self assert: self wrapperClass class instVarNames size = 0.
	self assert: self wrapperClass classVarNames size = 0.
.
	self assert: self wrapperClass protoMethods ~~ nil.
	self assert: self wrapperClass protoMethods class == Dictionary.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testException

	| wrapper instance |
	instance := MwClassA new.
	self should: [self result: instance methodWithException] raise: Warning.
	self assert: self result = 0.
	wrapper := self wrapperClass on: #methodWithException inClass: MwClassA.
	wrapper
		exception: Warning;
		handlerBlock: [:receiver :arguments :ex | 42];
		install.
	self shouldnt: [self result: instance methodWithException] raise: Warning.
	wrapper uninstall.
	self assert: self result = 42.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testInstallUninstall

	| wrapper |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	wrapper install.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == self wrapperClass.
	wrapper uninstall.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testNoException

	| wrapper instance |
	instance := MwClassA new.
	self shouldnt: [self result: instance methodWithoutException] raise: Warning.
	self assert: self result = 69.
	wrapper := self wrapperClass on: #methodWithoutException inClass: MwClassA.
	wrapper
		exception: Warning;
		handlerBlock: [:receiver :arguments :ex | 42];
		install.
	self shouldnt: [self result: instance methodWithoutException] raise: Warning.
	wrapper uninstall.
	self assert: self result = 69.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 01:02'!
testNoProtoInstanceCount

	| allInstCount protoInstCount |

	Smalltalk garbageCollect.
	allInstCount := self wrapperClass allInstances size.
	protoInstCount := self wrapperClass protoMethods size.
	self assert: (allInstCount - protoInstCount) = 0.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testOtherException

	| wrapper instance |
	instance := MwClassA new.
	self should: [instance methodWithException] raise: Warning.
	self assert: self result = 0.
	wrapper := self wrapperClass on: #methodWithException inClass: MwClassA.
	wrapper
		exception: Error;
		handlerBlock: [:receiver :arguments :ex | 42];
		install.
	self should: [self result: instance methodWithException] raise: Warning.
	wrapper uninstall.
	self assert: self result = 0.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'running' stamp: 'rhi 4/7/2001 21:51'!
testTweakClassFormat

	self assert: (self wrapperClass format bitAnd: 16r200) = 16r200.! !

!MwBlockHandlerMethodWrapperTest methodsFor: 'private' stamp: 'rhi 4/7/2001 21:41'!
wrapperClass

	^ MwBlockHandlerMethodWrapper! !

MwTestCase subclass: #MwBlockMethodWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'rhi 6/18/2001 14:32'!
setUp

	self result: 0.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testAfter

	| wrapper instance |
	instance := MwClassA new.
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapper
		afterBlock: [:receiver :arguments | self result: self result + 4];
		install.
	instance methodOne.
	wrapper uninstall.
	self assert: self result = 4.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testBefore

	| wrapper instance |
	instance := MwClassA new.
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapper
		beforeBlock: [:receiver :arguments | self result: self result + 3];
		install.
	instance methodOne.
	wrapper uninstall.
	self assert: self result = 3.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testBeforeAfter

	| wrapper instance |
	instance := MwClassA new.
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapper
		beforeBlock: [:receiver :arguments | self result: self result + 3];
		afterBlock: [:receiver :arguments | self result: self result + 4];
		install.
	instance methodOne.
	wrapper uninstall.
	self assert: self result = 7.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testBeforeAfterWithChangedArgument

	| wrapper instance |
	instance := MwClassA new.
	instance x: 0.
	wrapper := self wrapperClass on: #methodWithArgument: inClass: MwClassA.
	wrapper
		beforeBlock: [:receiver :arguments | arguments at: 1 put: (arguments first raisedTo: 4)];
		afterBlock: [:receiver :arguments | receiver x: receiver x + (arguments first raisedTo: 3)];
		install.
	self result: (instance methodWithArgument: 2).
	wrapper uninstall.
	self assert: self result = 16. "0 + 2^4"
	self assert: instance x = 4112. "0 + 2^4+ (2^4)^3"! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testBeforeAfterWithReceiver

	| wrapper instance |
	instance := MwClassA new.
	instance x: 0.
	wrapper := self wrapperClass on: #methodWithArgument: inClass: MwClassA.
	wrapper
		beforeBlock: [:receiver :arguments | receiver x: receiver x + 3];
		afterBlock: [:receiver :arguments | receiver x: receiver x + 4];
		install.
	self result: (instance methodWithArgument: 2).
	wrapper uninstall.
	self assert: self result = 5.
	self assert: instance x = 9.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testBeforeAfterWithReceiverAndArgument

	| wrapper instance |
	instance := MwClassA new.
	instance x: 0.
	wrapper := self wrapperClass on: #methodWithArgument: inClass: MwClassA.
	wrapper
		beforeBlock: [:receiver :arguments | receiver x: receiver x + (arguments first raisedTo: 2)];
		afterBlock: [:receiver :arguments | receiver x: receiver x + (arguments first raisedTo: 3)];
		install.
	self result: (instance methodWithArgument: 2).
	wrapper uninstall.
	self assert: self result = 6. "0 + 2^2 + 2"
	self assert: instance x = 14. "0 + 2^2 + 2 + 2^3"! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 10:00'!
testClassInstVars

	self assert: self wrapperClass class instVarNames size = 0.
	self assert: self wrapperClass classVarNames size = 0.

	self assert: self wrapperClass protoMethods ~~ nil.
	self assert: self wrapperClass protoMethods class == Dictionary.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testInstallUninstall

	| wrapper |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	wrapper install.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == self wrapperClass.
	wrapper uninstall.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 01:03'!
testNoProtoInstanceCount

	| allInstCount protoInstCount |

	Smalltalk garbageCollect.
	allInstCount := self wrapperClass allInstances size.
	protoInstCount := self wrapperClass protoMethods size.
	self assert: (allInstCount - protoInstCount) = 0.! !

!MwBlockMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 21:26'!
testTweakClassFormat

	self assert: (self wrapperClass format bitAnd: 16r200) = 16r200.! !

!MwBlockMethodWrapperTest methodsFor: 'private' stamp: 'rhi 3/29/2001 21:01'!
wrapperClass

	^ MwBlockMethodWrapper! !

MwTestCase subclass: #MwCalledMethodWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwCalledMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testCalledWasActive

	| wrapper instance |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapper install.
	instance := MwClassA new.
	self assert: wrapper called not.
	self assert: wrapper wasActive not.
	instance methodOne.
	self assert: wrapper wasActive.
	wrapper uninstall.! !

!MwCalledMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 10:00'!
testClassInstVars

	self assert: self wrapperClass class instVarNames size = 1.
	self assert: self wrapperClass classVarNames size = 0.

	self assert: self wrapperClass installing == false.

	self assert: self wrapperClass protoMethods ~~ nil.
	self assert: self wrapperClass protoMethods class == Dictionary.! !

!MwCalledMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 01:03'!
testNoProtoInstanceCount

	| allInstCount protoInstCount |

	Smalltalk garbageCollect.
	allInstCount := self wrapperClass allInstances size.
	protoInstCount := self wrapperClass protoMethods size.
	self assert: (allInstCount - protoInstCount) = 0.! !

!MwCalledMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 21:27'!
testTweakClassFormat

	self assert: (self wrapperClass format bitAnd: 16r200) = 16r200.! !

!MwCalledMethodWrapperTest methodsFor: 'private' stamp: 'rhi 3/29/2001 21:02'!
wrapperClass

	^ MwCalledMethodWrapper! !

MwTestCase subclass: #MwCountMethodWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwCountMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 09:58'!
testClassInstVars

	self assert: self wrapperClass class instVarNames size = 0.
	self assert: self wrapperClass classVarNames size = 0.

	self assert: self wrapperClass protoMethods ~~ nil.
	self assert: self wrapperClass protoMethods class == Dictionary.! !

!MwCountMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testCounts

	| wrapper instance |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapper install.
	instance := MwClassA new.
	self assert: wrapper count = 0.
	instance methodOne.
	self assert: wrapper count = 1.
	instance methodOne.
	self assert: wrapper count = 2.
	wrapper uninstall.! !

!MwCountMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 01:03'!
testNoProtoInstanceCount

	| allInstCount protoInstCount |

	Smalltalk garbageCollect.
	allInstCount := self wrapperClass allInstances size.
	protoInstCount := self wrapperClass protoMethods size.
	self assert: (allInstCount - protoInstCount) = 0.! !

!MwCountMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 21:17'!
testTweakClassFormat

	self assert: (self wrapperClass format bitAnd: 16r200) = 16r200.! !

!MwCountMethodWrapperTest methodsFor: 'private' stamp: 'rhi 3/29/2001 21:03'!
wrapperClass

	^ MwCountMethodWrapper! !

MwTestCase subclass: #MwMethodWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testArrayPosition0

	| expected actual |
	expected := 2.
	actual := (self wrapperClass createMethodFor: 0) arrayPosition.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testArrayPosition1

	| expected actual |
	expected := 2.
	actual := (self wrapperClass createMethodFor: 1) arrayPosition.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testArrayPosition2

	| expected actual |
	expected := 2.
	actual := (self wrapperClass createMethodFor: 2) arrayPosition.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'md 10/14/2004 18:58'!
testCanWrapInClass

	"Method wrappers can NOT be wrapped."
	self deny: (self wrapperClass canWrap: #spunk inClass: self wrapperClass).

	"BlockContext>>ensure: can NOT be wrapped."
	self deny: (self wrapperClass canWrap: #ensure: inClass: BlockContext).

	"CompiledMethod>>valueWithReceiver:arguments: can NOT be wrapped."
	self deny: (self wrapperClass
		canWrap: #valueWithReceiver:arguments: inClass: CompiledMethod).

	"A method that is not implemented by a class/object can NOT be wrapped."
	self assert: ((Object canUnderstand: #ensure:) not
		and: [(self wrapperClass canWrap: #ensure: inClass: Object) not]).

	"Certain primitives may NOT be wrapped."
	self assert: ((Object includesSelector: #at:put:)
		and: [(self wrapperClass canWrap: #at:put: inClass: Object) not]).
	self assert: ((BlockContext includesSelector: #value)
		and: [(self wrapperClass canWrap: #value inClass: BlockContext) not]).! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testChainRemoveLastFirst

	| wrapperOne wrapperTwo |
	wrapperOne := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapperTwo := self wrapperClass on: #methodOne inClass: MwClassA.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	wrapperOne install.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperOne clientMethod ~~ nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	wrapperTwo install.
	self assert: wrapperOne wrappingWrapper == wrapperTwo.
	self assert: wrapperTwo wrappingWrapper == nil.
	self assert: wrapperTwo clientMethod == wrapperOne.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperTwo.
	wrapperTwo uninstall.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperOne clientMethod ~~ nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	self assert: wrapperTwo clientMethod == nil.
	wrapperOne uninstall.
	self assert: wrapperOne clientMethod == nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 09:28'!
testChainRemoveLastFirstHierarchy
	| wrapperOne wrapperTwo |
	wrapperOne := self wrapperClass 
		on: #methodOne
		inClass: MwClassA.
	wrapperTwo := self wrapperClass 
		on: #methodOne
		inClass: MwClassB.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	wrapperOne install.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperOne clientMethod ~~ nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	wrapperTwo install.
	self assert: wrapperOne wrappingWrapper == wrapperTwo.
	self assert: wrapperTwo wrappingWrapper == nil.
	self assert: wrapperTwo clientMethod == wrapperOne.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	self assert: (MwClassB compiledMethodAt: #methodOne) == wrapperTwo.
	wrapperTwo uninstall.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperOne clientMethod ~~ nil.
	self assert: wrapperTwo clientMethod == nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	self assert: (MwClassB 
			compiledMethodAt: #methodOne
			ifAbsent: [ nil ]) == nil.
	wrapperOne uninstall.
	self assert: wrapperOne clientMethod == nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testChainRemoveMiddleFirst

	| wrapperOne wrapperTwo |
	wrapperOne := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapperTwo := self wrapperClass on: #methodOne inClass: MwClassA.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	wrapperOne install.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperOne clientMethod ~~ nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	wrapperTwo install.
	self assert: wrapperOne wrappingWrapper == wrapperTwo.
	self assert: wrapperTwo wrappingWrapper == nil.
	self assert: wrapperTwo clientMethod == wrapperOne.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperTwo.
	wrapperOne uninstall.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperTwo clientMethod ~~ nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperTwo.
	self assert: wrapperOne clientMethod == nil.
	wrapperTwo uninstall.
	self assert: wrapperTwo clientMethod == nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 09:28'!
testChainRemoveMiddleFirstHierarchy
	| wrapperOne wrapperTwo |
	wrapperOne := self wrapperClass 
		on: #methodOne
		inClass: MwClassA.
	wrapperTwo := self wrapperClass 
		on: #methodOne
		inClass: MwClassB.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	wrapperOne install.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperOne clientMethod ~~ nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	wrapperTwo install.
	self assert: wrapperOne wrappingWrapper == wrapperTwo.
	self assert: wrapperTwo wrappingWrapper == nil.
	self assert: wrapperTwo clientMethod == wrapperOne.
	self assert: (MwClassA compiledMethodAt: #methodOne) == wrapperOne.
	self assert: (MwClassB compiledMethodAt: #methodOne) == wrapperTwo.
	wrapperOne uninstall.
	self assert: wrapperOne wrappingWrapper == nil.
	self assert: wrapperOne clientMethod == nil.
	self assert: wrapperTwo clientMethod ~~ nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	self assert: (MwClassB compiledMethodAt: #methodOne) == wrapperTwo.
	wrapperTwo uninstall.
	self assert: wrapperTwo clientMethod == nil.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	self assert: (MwClassB 
			compiledMethodAt: #methodOne
			ifAbsent: [ nil ]) == nil! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/27/2009 20:45'!
testClassInstVars

	self assert: self wrapperClass classVarNames size = 0.

	self assert: self wrapperClass protoMethods ~~ nil.
	self assert: self wrapperClass protoMethods class == Dictionary.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 20:53'!
testCodeString0

	| expected actual |
	expected _ 'value
	| t |
	t _ #Array new.
	^ #''The method wrapper should be inserted in this position!!'' valueWithReceiver: self arguments: t'.
	actual _ self wrapperClass codeStringFor: 0.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 20:53'!
testCodeString1

	| expected actual |
	expected _ 'value: t1
	| t |
	(t _ #Array new: 1) at: 1 put: t1.
	^ #''The method wrapper should be inserted in this position!!'' valueWithReceiver: self arguments: t'.
	actual _ self wrapperClass codeStringFor: 1.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 20:53'!
testCodeString2

	| expected actual |
	expected _ 'value: t1 value: t2
	| t |
	(t _ #Array new: 2) at: 1 put: t1; at: 2 put: t2.
	^ #''The method wrapper should be inserted in this position!!'' valueWithReceiver: self arguments: t'.
	actual _ self wrapperClass codeStringFor: 2.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testDefiningWrappedDifferent

	| wrapper clientSelector wrappedClass |
	clientSelector := #methodOne.
	wrappedClass := MwClassB.
	wrapper := self wrapperClass on: clientSelector inClass: wrappedClass.
	wrapper install.
	self assert: wrapper definingClass == MwClassA.
	self assert: wrapper wrappedClass == wrappedClass.
	wrapper uninstall.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testDefiningWrappedSame

	| wrapper clientSelector wrappedClass |
	clientSelector := #methodOne.
	wrappedClass := MwClassA.
	wrapper := self wrapperClass on: clientSelector inClass: wrappedClass.
	wrapper install.
	self assert: wrapper definingClass == wrappedClass.
	self assert: wrapper wrappedClass == wrappedClass.
	wrapper uninstall.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testDefiningWrappedSame2

	| wrapper clientSelector wrappedClass |
	clientSelector := #methodTwo.
	wrappedClass := MwClassA.
	wrapper := self wrapperClass on: clientSelector inClass: wrappedClass.
	wrapper install.
	self assert: wrapper definingClass == wrappedClass.
	self assert: wrapper wrappedClass == wrappedClass.
	wrapper uninstall.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testDefiningWrappedSame3

	| wrapper clientSelector wrappedClass |
	clientSelector := #methodTwo.
	wrappedClass := MwClassB.
	wrapper := self wrapperClass on: clientSelector inClass: wrappedClass.
	wrapper install.
	self assert: wrapper definingClass == wrappedClass.
	self assert: wrapper wrappedClass == wrappedClass.
	wrapper uninstall.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testDefiningWrappedSame4

	| wrapper clientSelector wrappedClass |
	clientSelector := #methodThree.
	wrappedClass := MwClassB.
	wrapper := self wrapperClass on: clientSelector inClass: wrappedClass.
	wrapper install.
	self assert: wrapper definingClass == wrappedClass.
	self assert: wrapper wrappedClass == wrappedClass.
	wrapper uninstall.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testInstallInstantiateUninstall

	| wrapper instance |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	wrapper install.
	instance := MwClassA new.
	instance methodOne.
	wrapper uninstall.
	instance methodOne.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testInstallUninstall

	| wrapper |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.
	wrapper install.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == self wrapperClass.
	wrapper uninstall.
	self assert: (MwClassA compiledMethodAt: #methodOne) class == CompiledMethod.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testInstantiateInstallUninstall

	| wrapper instance |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	instance := MwClassA new.
	instance methodOne.
	wrapper install.
	instance methodOne.
	wrapper uninstall.
	instance methodOne.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testMethodPosition0

	| expected actual |
	expected := 4.
	actual := (self wrapperClass createMethodFor: 0) methodPosition.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testMethodPosition1

	| expected actual |
	expected := 4.
	actual := (self wrapperClass createMethodFor: 1) methodPosition.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testMethodPosition2

	| expected actual |
	expected := 4.
	actual := (self wrapperClass createMethodFor: 2) methodPosition.
	self assert: expected = actual.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/27/2009 20:53'!
testNoProtoInstanceCount

	| allInstCount protoInstCount |

	Smalltalk garbageCollect.
	allInstCount := self wrapperClass allInstances size.
	protoInstCount := self wrapperClass protoMethods size.
	self assert: (allInstCount - protoInstCount) = 0.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testOnInClass

	| wrapper clientSelector wrappedClass |
	clientSelector := #methodOne.
	wrappedClass := MwClassA.
	wrapper := self wrapperClass on: clientSelector inClass: wrappedClass.
	self assert: wrapper clientSelector == clientSelector.
	self assert: wrapper clientMethod == nil.
	self assert: wrapper definingClass == nil.
	self assert: wrapper wrappedClass == wrappedClass.
	self assert: wrapper wrappingWrapper == nil.
	self assert: (wrapper realObjectAt: wrapper methodPosition) == wrapper.
	self assert: (wrapper realObjectAt: wrapper arrayPosition) == Array.
	wrapper uninstall.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/27/2009 20:40'!
testProperties
	| wrapper |
	wrapper := self wrapperClass createMethodFor: 0.
	self assert: wrapper clientSelector == nil.
	self assert: wrapper clientMethod == nil.
	self assert: wrapper definingClass == nil.
	self assert: wrapper wrappedClass == nil.
	self assert: wrapper wrappingWrapper == nil! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 21:27'!
testTweakClassFormat

	self assert: (self wrapperClass format bitAnd: 16r200) = 16r200.! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 21:30'!
testTweakClassFormatAllSubclasses

	self wrapperClass withAllSubclassesDo: [:each |
		self assert: (each format bitAnd: 16r200) = 16r200].! !

!MwMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testWasActive

	| wrapper |
	wrapper := self wrapperClass on: #methodOne inClass: MwClassA.
	self deny: wrapper wasActive.
	wrapper install.
	self deny: wrapper wasActive.
	wrapper uninstall.
	self assert: wrapper wasActive.
	self
		should: [wrapper install]
		raise: Error.! !

!MwMethodWrapperTest methodsFor: 'private' stamp: 'rhi 3/29/2001 20:50'!
wrapperClass

	^ MwMethodWrapper! !

MwTestCase subclass: #MwSynchronizationWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwSynchronizationWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 10:00'!
testClassInstVars

	self assert: self wrapperClass class instVarNames size = 0.
	self assert: self wrapperClass classVarNames size = 0.

	self assert: self wrapperClass protoMethods ~~ nil.
	self assert: self wrapperClass protoMethods class == Dictionary.! !

!MwSynchronizationWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 01:03'!
testNoProtoInstanceCount

	| allInstCount protoInstCount |

	Smalltalk garbageCollect.
	allInstCount := self wrapperClass allInstances size.
	protoInstCount := self wrapperClass protoMethods size.
	self assert: (allInstCount - protoInstCount) = 0.! !

!MwSynchronizationWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 21:27'!
testTweakClassFormat

	self assert: (self wrapperClass format bitAnd: 16r200) = 16r200.! !

!MwSynchronizationWrapperTest methodsFor: 'private' stamp: 'rhi 3/29/2001 21:03'!
wrapperClass

	^ MwSynchronizationWrapper! !

!MwTestCase methodsFor: 'accessing' stamp: 'rhi 6/18/2001 14:31'!
result
	"^ <Integer>"

	^ result! !

!MwTestCase methodsFor: 'accessing' stamp: 'DF 4/19/2006 00:00'!
result: anInteger

	result := anInteger.! !

MwTestCase subclass: #MwTimeMethodWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodWrappers-Tests'!

!MwTimeMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testAverageTime

	| wrapper instance count |
	count := 10.
	wrapper := self wrapperClass on: #methodDelay inClass: MwClassA.
	wrapper install.
	instance := MwClassA new.
	count timesRepeat: [instance methodDelay].
	self assert: wrapper count = count.
	self assert: (wrapper time / count asFloat) = wrapper averageTime.
	wrapper uninstall.! !

!MwTimeMethodWrapperTest methodsFor: 'running' stamp: 'bs 10/16/2009 10:01'!
testClassInstVars

	self assert: self wrapperClass class instVarNames size = 0.
	self assert: self wrapperClass classVarNames size = 0.

	self assert: self wrapperClass protoMethods ~~ nil.
	self assert: self wrapperClass protoMethods class == Dictionary.! !

!MwTimeMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testCounts

	| wrapper instance |
	wrapper := self wrapperClass on: #methodDelay inClass: MwClassA.
	wrapper install.
	instance := MwClassA new.
	self assert: wrapper count = 0.
	instance methodDelay.
	self assert: wrapper count = 1.
	instance methodDelay.
	self assert: wrapper count = 2.
	wrapper uninstall.! !

!MwTimeMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testCountsAndTimes

	| wrapper instance |
	wrapper := self wrapperClass on: #methodDelay inClass: MwClassA.
	wrapper install.
	instance := MwClassA new.
	self assert: wrapper count = 0.
	self assert: wrapper time = 0.
	instance methodDelay.
	self assert: wrapper count = 1.
	self assert: wrapper time > 0.
	instance methodDelay.
	self assert: wrapper count = 2.
	self assert: wrapper time > 0.
	wrapper uninstall.! !

!MwTimeMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 01:03'!
testNoProtoInstanceCount

	| allInstCount protoInstCount |

	Smalltalk garbageCollect.
	allInstCount := self wrapperClass allInstances size.
	protoInstCount := self wrapperClass protoMethods size.
	self assert: (allInstCount - protoInstCount) = 0.! !

!MwTimeMethodWrapperTest methodsFor: 'running' stamp: 'DF 4/19/2006 00:00'!
testTimes

	| wrapper instance previousTime |
	wrapper := self wrapperClass on: #methodDelay inClass: MwClassA.
	wrapper install.
	instance := MwClassA new.
	self assert: wrapper time = 0.
	previousTime := wrapper time.
	instance methodDelay.
	self assert: wrapper time > previousTime.
	previousTime := wrapper time.
	instance methodDelay.
	self assert: wrapper time > previousTime.
	wrapper uninstall.! !

!MwTimeMethodWrapperTest methodsFor: 'running' stamp: 'rhi 3/29/2001 21:27'!
testTweakClassFormat

	self assert: (self wrapperClass format bitAnd: 16r200) = 16r200.! !

!MwTimeMethodWrapperTest methodsFor: 'private' stamp: 'rhi 3/29/2001 21:04'!
wrapperClass

	^ MwTimeMethodWrapper! !
MwBlockHandlerMethodWrapper initialize!
MwBlockMethodWrapper initialize!
MwCalledMethodWrapper initialize!
MwCountMethodWrapper initialize!
MwTimeMethodWrapper initialize!
MwMethodWrapper initialize!
MwSynchronizationWrapper initialize!
